import datetime
import typing


class Person:
    def __init__(self, identifier: str):
        self.identifier: str = identifier
        self.name: typing.Optional[str] = None
        self.birth_date: typing.Optional[datetime.date] = None
        self.mother: typing.Optional["Person"] = None
        self.father: typing.Optional["Person"] = None

    def __repr__(self) -> str:
        """
        >>> p = Person("max-mustermann")
        >>> repr(p)
        'Person(max-mustermann)'
        >>> p.name = "Hr. Mustermann"
        >>> repr(p)
        'Person(max-mustermann, Hr. Mustermann)'
        >>> p.name = "Max Mustermann"
        >>> repr(p)
        'Person(max-mustermann, Max Mustermann)'
        >>> p.birth_date = datetime.date(1976, 2, 1)
        >>> repr(p)
        'Person(max-mustermann, Max Mustermann, *1976-02-01)'
        """
        return "{}({})".format(
            type(self).__name__,
            ", ".join(
                filter(
                    None,
                    (
                        self.identifier,
                        self.name,
                        "*" + self.birth_date.isoformat()
                        if self.birth_date is not None
                        else None,
                    ),
                )
            ),
        )

    def __str__(self) -> str:
        """
        >>> p = Person("max-mustermann")
        >>> p.name = "Max Mustermann"
        >>> str(p)
        'Max Mustermann'
        >>> p.birth_date = datetime.date(1976, 2, 1)
        >>> str(p)
        'Max Mustermann (*1976-02-01)'
        """
        return (self.name or "unnamed") + (
            " (*{})".format(self.birth_date.isoformat()) if self.birth_date else ""
        )

    def __eq__(self, other: "Person") -> bool:
        """
        >>> maxl = Person("max")
        >>> maxl.name = "Max Mustermann"
        >>> maxl == Person("max")
        True
        >>> erika = Person("erika")
        >>> erika.name = "Max Mustermann"
        >>> maxl == erika
        False
        """
        return self.identifier == other.identifier

    def merge(self, person: "Person") -> None:
        """
        >>> p1 = Person("max")
        >>> p1.name = "Max Mustermann"
        >>> str(p1)
        'Max Mustermann'
        >>> p2 = Person("max2")
        >>> p2.birth_date = datetime.date(1976, 2, 1)
        >>> p2.mother = Person("mother")
        >>> p2.father = Person("father")
        >>> str(p2)
        'unnamed (*1976-02-01)'

        add attributes of p2 to p1:
        >>> p1.merge(p2)
        >>> str(p1)
        'Max Mustermann (*1976-02-01)'
        >>> p1.mother, p1.father
        (Person(mother), Person(father))

        p2 is unchanged:
        >>> str(p2)
        'unnamed (*1976-02-01)'
        """
        for attr in ["name", "birth_date", "mother", "father"]:
            if getattr(person, attr) is not None:
                setattr(self, attr, getattr(person, attr))
